<?php

/**
 * @file
 * Provides support for the Views module.
 */

/**
 * Implementation of hook_views_tables
 */
function views_bookmark_views_tables() {
  $bookmarks = views_bookmark_get_bookmarks();
  foreach ($bookmarks as $bookmark) {
    // Localize the bookmark title:
    $bookmark = _views_bookmark_process_labels($bookmark, NULL, array('title'));
    $table = array(
      'name' => 'views_bookmark_nodes',
      'join' => array(
        'left' => array(
          'table' => 'node',
          'field' => 'nid'
        ),
        'right' => array(
          'field' => 'nid'
        ),
        'extra' => array(
          'vbid' => $bookmark->vbid
        ),
      ),
      'fields' => array(
        'timestamp' => array(
          'name' => t('Bookmark: @s Marked Time', array('@s' => $bookmark->title)),
          'sortable' => TRUE,
          'handler' => views_handler_field_dates(),
          'option' => 'string',
          'help' => t('Display the time the node was bookmarked by a user.')
                  .' '. t('The option field may be used to specify the custom date format as it\'s required by the date() function or if "as time ago" has been chosen to customize the granularity of the time interval.'),
        ),
      ),
      'filters' => array(
        'uid' => array(
          'name' => t('Bookmark: @s', array('@s' => $bookmark->title)),
          // Do not be concerned about reversal here. IS NOT NULL means marked.
          'operator' => array('IS NOT' => 'Is Marked By'),
          'list' => array('***CURRENT_USER***' => t('Currently Logged In User'), 'uid' => t('Any User')),
          'list-type' => 'select',
          'handler' => 'views_bookmark_filter_handler_uid',
          'help' => t('This will filter a only nodes that have the %s mark. If using <em>Any User</em>, often the <em>Node: Distinct</em> filter will also be desired. The <em>Any User</em> filter is not necessary if combining with other bookmark sorts or arguments.', array('%s' => $bookmark->title)),
          'vbid' => $bookmark->vbid,
        ),
        'timestamp' => array(
          'name' => t('Bookmark: @s Marked Time', array('@s' => $bookmark->title)),
          'operator' => 'views_handler_operator_gtlt',
          'value' => views_handler_filter_date_value_form(),
          'handler' => 'views_handler_filter_timestamp',
          'option' => 'string',
          'help' => t('This filter allows nodes to be filtered by the time they were marked by a user.')
            .' '. views_t_strings('filter date'),
        ),
      ),
      'sorts' => array(
        'timestamp' => array(
          'name' => t('Bookmark: @s Marked Time', array('@s' => $bookmark->title)),
          'help' => t('Sort by the time a node was bookmarked using the %s mark.', array('%s' => $bookmark->title)),
        ),
      ),
    );
    $tables["views_bookmark_nodes_$bookmark->vbid"] = $table;

    $table = array(
      'name' => 'users',
      'join' => array(
        'left' => array(
          'table' => "views_bookmark_nodes_$bookmark->vbid",
          'field' => 'uid',
        ),
        'right' => array(
          'field' => 'uid'
        ),
      ),
      'fields' => array(
        'name' => array(
          'name' => t('Bookmark: User for @s', array('@s' => $bookmark->title)),
          'handler' => 'views_handler_field_username',
          'sortable' => TRUE,
          'help' => t('This will display the user who bookmarked the node.'),
          'uid' => 'uid',
          'addlfields' => array('uid'),
        ),
      ),
      'sorts' => array(
        'name' => array(
          'name' => t('Bookmark: User for @s', array('@s' => $bookmark->title)),
          'help' => t('Sort by the the name of the users who bookmarked the node using the %s mark. This probably makes sense only when the list is filtered for a single node; e.g., on a "Who\'s bookmarked this" tab.', array('%s' => $bookmark->title)),
        ),
      ),
    );
    $tables["views_bookmark_users_$bookmark->vbid"] = $table;

    $table = array(
      'name' => 'views_bookmark_nodetypes',
      'provider' => 'views_bookmark',
      'join' => array(
        'left' => array(
          'table' => 'node',
          'field' => 'type'
        ),
        'right' => array(
          'field' => 'type'
        ),
        'extra' => array(
          'vbid' => $bookmark->vbid
        ),
      ),
    );
    $tables["views_bookmark_nodetypes_$bookmark->vbid"] = $table;

    $table = array(
      'name' => 'views_bookmark_node_count',
      'provider' => 'views_bookmark',
      'join' => array(
        'left' => array(
          'table' => 'node',
          'field' => 'nid'
        ),
        'right' => array(
          'field' => 'nid'
        ),
        'extra' => array(
          'vbid' => $bookmark->vbid
        ),
      ),
      'fields' => array(
        'count' => array(
          'name' => t('Bookmark: Count for @s', array('@s' => $bookmark->title)),
          'sortable' => true,
          'handler' => array(
            'views_handler_field_int' => t('Display the count as an integer'),
            'views_bookmark_field_handler_count' => t('Display the count in a phrase'),
          ),
          'option' => 'string',
          'help' => t('If you choose <em>Display the count in a phrase</em>, use the Option field plus the <code>@count</code> placeholder to craft your phrase. A pipe character may separate the singular form from the plural form. Another pipe may separate a leading zero phrase --which may well be an empty string-- from the aforementioned two. Examples: <code>"@count", "1 vote|@count votes", "needs voting|1 vote|@count votes", "|1 vote|@count votes", "|@count|@count"</code>.'),
        ),
      ),
      'filters' => array(
        'count' => array(
          'name' => t('Bookmark: Count for @s', array('@s' => $bookmark->title)),
          'help' => t('This will filter by the number of times that a node has been been bookmarked.'),
          'operator' => 'views_handler_operator_gtlt',
          'option' => 'integer',
        ),
      ),
      'sorts' => array(
        'count' => array(
          'name' => t('Bookmark: @s Count', array('@s' => $bookmark->title)),
          'help' => t('Sort by the total bookmark count for each node.'),
        ),
      ),
    );
    $tables["views_bookmark_node_count_$bookmark->vbid"] = $table;

    $table = array(
      'name' => 'views_bookmark_nodes',
      'join' => array(
        'left' => array(
          'table' => 'node',
          'field' => 'nid',
        ),
        'right' => array(
          'field' => 'nid',
        ),
        'extra' => array(
          'vbid' => $bookmark->vbid,
          'uid' => $bookmark->global ? 0 : '***CURRENT_USER***',
          // Note: since this is a LEFT JOIN, we also get records where
          // uid is empty; that is, nodes which haven't been bookmarked.
          // This is intentional, as we want to display the Ops field
          // for unbookmarked nodes too.
          // The Ops field handler utilizes uid being empty (NULL) to
          // efficiently figure out whether or not a node is bookmarked.
        ),
      ),
      'fields' => array(
        'ops' => array(
          'field' => 'uid',
          'name' => t('Bookmark: Ops for @s', array('@s' => $bookmark->title)),
          'help' => t('Display an AJAX link to bookmark, or unbookmark, the node. Note that the operation is made on behalf of the user who <em>browses the page</em> and not the user whose name happens to appear adjacent to the link.'),
          'handler' => 'views_bookmark_field_handler_ops',
          'query_handler' => 'views_bookmark_query_handler_ops',
          'vbid' => $bookmark->vbid,
        ),
      ),
    );
    $tables["views_bookmark_ops_$bookmark->vbid"] = $table;
  }

  return $tables;
}

/**
 * Handler that prints the proper operations for a bookmark.
 */
function views_bookmark_field_handler_ops($fieldinfo, $fielddata, $value, $data) {
  $vbid = $fieldinfo['vbid'];

  $node_type_field = 'views_bookmark_nodetypes_'. $vbid .'_type';
  if (!$data->$node_type_field) {
    // bookmark does not apply to this node.
    return;
  }

  $bookmarks = views_bookmark_get_bookmarks(NULL, $GLOBALS['user']);
  if (!isset($bookmarks[$vbid])) {
    // bookmark does not apply to this user.
    return;
  }
  $bookmark = $bookmarks[$vbid];

  // Token replacements.
  $bookmark = _views_bookmark_process_labels($bookmark, node_load($data->nid), array('mark', 'mark_long'));

  $mark_html = l($bookmark->mark, "views_bookmark/mark/$data->nid/$vbid", array('class' => "views-bookmark-mark links_bookmark_$vbid", 'title' => $bookmark->mark_long), "destination=$_GET[q]");
  $unmark_html = l($bookmark->unmark, "views_bookmark/unmark/$data->nid/$vbid", array('class' => "views-bookmark-unmark links_bookmark_$vbid", 'title' => $bookmark->unmark_long), "destination=$_GET[q]");

  views_bookmark_add_js($data->nid, $vbid, $mark_html, $unmark_html);

  // The value of this field is NULL if there isn't a record for this
  // node in the 'views_bookmark_nodes' table. That is, value is NULL if
  // this node isn't bookmarked.
  if (!is_null($value)) {
    return $unmark_html;
  }
  else {
    return $mark_html;
  }
}

/**
 * This handler loads the type of the node so the display handler, above,
 * can determine if the bookmark applies to it.
 */
function views_bookmark_query_handler_ops($field, $fieldinfo, &$query) {
  $vbid = $fieldinfo['vbid'];

  // Load the type of the node.
  // I could simply do $query->add_field('type', 'node', 'node_type') and get
  // rid of this pseudo table, defined in hook_views_tables(), but I suspect
  // that it will be easier to port to Views 2 if we don't hardcode the 'node'.
  $query->add_field('type', "views_bookmark_nodetypes_$vbid", "views_bookmark_nodetypes_$vbid" .'_type');
}

/**
 * Handler that lets us filter nodes to include only those by a particular user.
 */
function views_bookmark_filter_handler_uid($op, $filter, $filterinfo, &$query) {
  // Load the views bookmark to check if it's global.
  $bookmark = views_bookmark_get_bookmark($filterinfo['vbid']);

  $tn = $query->add_table($filterinfo['table']);
  $tname = $query->get_table_name($filterinfo['table'], $tn);
  $query->add_where("$tname.uid $filter[operator] NULL");
  if ($filter['value'] == '***CURRENT_USER***' && !$bookmark->global) {
    $query->add_where("$tname.uid = ***CURRENT_USER***");
  }
}

/**
 * Handler that prints the 'count' field in a zero/singular/plural phrase, localizable.
 */
function views_bookmark_field_handler_count($fieldinfo, $fielddata, $value, $data) {
  return _views_bookmark_format_plural($value, $fielddata['options']);
}

/**
 * Give an argument handler to limit nodes to those bookmarked by a uid.
 */
function views_bookmark_views_arguments() {
  $bookmarks = views_bookmark_get_bookmarks();
  foreach ($bookmarks as $bookmark) {
    // Localize the bookmark title:
    $bookmark = _views_bookmark_process_labels($bookmark, NULL, array('title'));
    $argument = array(
      'name' => t("Bookmarks '@s' for UID", array('@s' => $bookmark->title)),
      'handler' => "views_bookmark_arg_uid",
      'help' => t("This argument allows users to filter a view to nodes marked with the '%s' bookmark for the specified user.", array("%s" => $bookmark->title)),
      'table' => "views_bookmark_nodes_$bookmark->vbid",
    );
    $arguments["views_bookmark_nodes_$bookmark->vbid"] = $argument;
  }
  return $arguments;
}

function views_bookmark_arg_uid($op, &$query, $argtype, $arg = '') {
  switch ($op) {
    case 'summary':
      // We have to do a couple of kind of ugly things here. First,
      // because we're dynamic we dig some table info out of the
      // arguments array. Second, the default behavior on our
      // pseudotables is to lock the table to current user, but
      // we are overriding that.

      $argdata = _views_get_arguments();
      $table = $argdata[$argtype]['table'];

      $table_data = _views_get_tables();
      $joininfo = $table_data[$table]['join'];
      unset($joininfo['extra']);

      $query->add_table($table, true, 1, $joininfo);
      $query->add_table('users', true, 1, array('left' => array('table' => $table, 'field' => 'uid'), 'right' => array('field' => 'uid')));
      $query->add_field('name', 'users');
      $query->add_field('uid', 'users');
      $query->add_where("$table.uid IS NOT NULL");
      $fieldinfo['field'] = "users.name";
      return $fieldinfo;
      break;
    case 'filter':
      $uid = intval($arg);
      $argdata = _views_get_arguments();
      $table = $argdata[$argtype['type']]['table'];

      $table_data = _views_get_tables();
      $joininfo = $table_data[$table]['join'];
      $joininfo['extra']['uid'] = $uid;

      $query->add_table($table, true, 1, $joininfo);
      $query->add_where("$table.uid IS NOT NULL");
      break;
    case 'link':
      $name = ($query->name ? $query->name : variable_get('anonymous', t('Anonymous')));
      return l($name, "$arg/" . intval($query->uid));
    case 'title':
      if (!$query) {
        return variable_get('anonymous', t('Anonymous'));
      }
      $user = db_fetch_object(db_query("SELECT name FROM {users} WHERE uid = '%d'", $query));
      return $user->name;
  }
}

/**
 * Implementation of hook_views_default_views()
 */
function views_bookmark_views_default_views() {
  $bookmarks = views_bookmark_get_bookmarks();

  foreach ($bookmarks as $bookmark) {
    // Localize the bookmark title:
    $bookmark = _views_bookmark_process_labels($bookmark, NULL, array('title'));
    $vbid = $bookmark->vbid;
    $view = new stdClass();
    $view->name = "bookmarks_$vbid"; // 'title' isn't actually safe here
    $view->access = empty($bookmark->roles) ? array(DRUPAL_AUTHENTICATED_RID) : $bookmark->roles;
    $view->description = t('View for bookmark: !bookmark-title', array('!bookmark-title' => $bookmark->title));
    $view->page = TRUE;
    $view->page_title = t('My !bookmark-title', array('!bookmark-title' => $bookmark->title));
    $view->page_type = 'table';
    $view->url = "bookmarks/$vbid";
    $view->use_pager = TRUE;
    $view->nodes_per_page = '25';
    $view->menu = TRUE;
    $view->menu_tab = FALSE;
    $view->menu_tab_default = FALSE;
    $view->sort = array (
    );
    $view->argument = array (
    );
    $view->field = array (
      array (
        'tablename' => 'node',
        'field' => 'type',
        'label' => 'Type',
        'sortable' => '1',
      ),
      array (
        'tablename' => 'node',
        'field' => 'title',
        'label' => 'Title',
        'handler' => 'views_handler_field_nodelink_with_mark',
        'sortable' => '1',
      ),
      array (
        'tablename' => 'users',
        'field' => 'name',
        'label' => 'Author',
        'sortable' => '1',
      ),
      array (
        'tablename' => "views_bookmark_ops_$vbid",
        'field' => 'ops',
        'label' => 'Ops',
      ),
    );
    $view->filter = array (
      array (
        'tablename' => 'node',
        'field' => 'status',
        'operator' => '=',
        'options' => '',
        'value' => '1',
      ),
      array (
        'tablename' => "views_bookmark_nodes_$vbid",
        'field' => 'uid',
        'operator' => 'IS NOT',
        'options' => '',
        'value' => '***CURRENT_USER***',
      ),
    );
    $view->requires = array('node', 'users');

    // If comment module exists, append two additional fields to the default
    // view. If comment module is added later, these fields will be added.
    if (module_exists('comment')) {
      $op_field = array_pop($view->field);
      $view->field = array_merge($view->field, array (
        array (
          'tablename' => 'node_comment_statistics',
          'field' => 'comment_count',
          'label' => 'Replies',
          'handler' => 'views_handler_comments_with_new',
          'sortable' => '1',
        ),
        array (
          'tablename' => 'node_comment_statistics',
          'field' => 'last_comment_timestamp',
          'label' => 'Last Post',
          'handler' => 'views_handler_field_since',
          'sortable' => '1',
          'defaultsort' => 'DESC',
        ),
      ));
      array_push($view->field, $op_field);
      $view->requires = array_merge($view->requires, array('node_comment_statistics'));
    }

    $views[$view->name] = $view;
  }
  return $views;
}

